@@ -1,6 +1,6 @@
 #!/usr/bin/python
 '''
-Backend to the console plugin.
+Provides IPython console widget.
 
 @author: Eitan Isaacson
 @organization: IBM Corporation
@@ -24,26 +24,43 @@ try:
 except ImportError:
   IPython = None
 
-ansi_colors =  {'0;30': 'Black',
-                '0;31': 'Red',
-                '0;32': 'Green',
-                '0;33': 'Brown',
-                '0;34': 'Blue',
-                '0;35': 'Purple',
-                '0;36': 'Cyan',
-                '0;37': 'LightGray',
-                '1;30': 'DarkGray',
-                '1;31': 'DarkRed',
-                '1;32': 'SeaGreen',
-                '1;33': 'Yellow',
-                '1;34': 'LightBlue',
-                '1;35': 'MediumPurple',
-                '1;36': 'LightCyan',
-                '1;37': 'White'}
-
 class IterableIPShell:
+  '''
+  Create an IPython instance. Does not start a blocking event loop,
+  instead allow single iterations. This allows embedding in GTK+ 
+  without blockage.
+
+  @ivar IP: IPython instance.
+  @type IP: IPython.iplib.InteractiveShell
+  @ivar iter_more: Indicates if the line executed was a complete command,
+  or we should wait for more.
+  @type iter_more: integer
+  @ivar history_level: The place in history where we currently are 
+  when pressing up/down.
+  @type history_level: integer
+  @ivar complete_sep: Seperation delimeters for completion function.
+  @type complete_sep: _sre.SRE_Pattern
+  '''
   def __init__(self,argv=[],user_ns=None,user_global_ns=None, 
                cin=None, cout=None,cerr=None, input_func=None):
+    '''
+    
+    
+    @param argv: Command line options for IPython
+    @type argv: list
+    @param user_ns: User namespace.
+    @type user_ns: dictionary
+    @param user_global_ns: User global namespace.
+    @type user_global_ns: dictionary.
+    @param cin: Console standard input.
+    @type cin: IO stream
+    @param cout: Console standard output.
+    @type cout: IO stream 
+    @param cerr: Console standard error.
+    @type cerr: IO stream
+    @param input_func: Replacement for builtin raw_input()
+    @type input_func: function
+    '''
     if input_func:
       IPython.iplib.raw_input_original = input_func
     if cin:
@@ -60,10 +77,11 @@ class IterableIPShell:
     self.term = IPython.genutils.IOTerm(cin=cin, cout=cout, cerr=cerr)
     os.environ['TERM'] = 'dumb'
     excepthook = sys.excepthook 
-    self.IP = IPython.Shell.make_IPython(argv,user_ns=user_ns,
-                                         user_global_ns=user_global_ns,
-                                         embedded=True,
-                                         shell_class=IPython.Shell.InteractiveShell)
+    self.IP = IPython.Shell.make_IPython(
+      argv,user_ns=user_ns,
+      user_global_ns=user_global_ns,
+      embedded=True,
+      shell_class=IPython.Shell.InteractiveShell)
     self.IP.system = lambda cmd: self.shell(self.IP.var_expand(cmd),
                                             header='IPython system call: ',
                                             verbose=self.IP.rc.system_verbose)
@@ -73,6 +91,9 @@ class IterableIPShell:
     self.complete_sep =  re.compile('[\s\{\}\[\]\(\)]')
 
   def execute(self):
+    '''
+    Executes the current line provided by the shell object.
+    '''
     self.history_level = 0
     orig_stdout = sys.stdout
     sys.stdout = IPython.Shell.Term.cout
@@ -105,14 +126,32 @@ class IterableIPShell:
     sys.stdout = orig_stdout
 
   def historyBack(self):
+    '''
+    Provides one history command back.
+    
+    @return: The command string.
+    @rtype: string
+    '''
     self.history_level -= 1
     return self._getHistory()
   
   def historyForward(self):
+    '''
+    Provides one history command forward.
+    
+    @return: The command string.
+    @rtype: string
+    '''
     self.history_level += 1
     return self._getHistory()
   
   def _getHistory(self):
+    '''
+    Get's the command string of the current history level.
+    
+    @return: Historic command string.
+    @rtype: string
+    '''
     try:
       rv = self.IP.user_ns['In'][self.history_level].strip('\n')
     except IndexError:
@@ -121,25 +160,64 @@ class IterableIPShell:
     return rv
 
   def updateNamespace(self, ns_dict):
+    '''
+    Add the current dictionary to the shell namespace.
+    
+    @param ns_dict: A dictionary of symbol-values.
+    @type ns_dict: dictionary
+    '''
     self.IP.user_ns.update(ns_dict)
 
   def complete(self, line):
+    '''
+    Returns an auto completed line and/or posibilities for completion.
+    
+    @param line: Given line so far.
+    @type line: string
+    
+    @return: Line completed as for as possible, 
+    and possible further completions.
+    @rtype: tuple
+    '''
     split_line = self.complete_sep.split(line)
     possibilities = self.IP.complete(split_line[-1])
     if possibilities:
-      common_prefix = reduce(self._commonPrefix, possibilities)
+      def _commonPrefix(str1, str2):
+        '''
+        Reduction function. returns common prefix of two given strings.
+        
+        @param str1: First string.
+        @type str1: string
+        @param str2: Second string
+        @type str2: string
+        
+        @return: Common prefix to both strings.
+        @rtype: string
+        '''
+        for i in range(len(str1)):
+          if not str2.startswith(str1[:i+1]):
+            return str1[:i]
+        return str1
+      common_prefix = reduce(_commonPrefix, possibilities)
       completed = line[:-len(split_line[-1])]+common_prefix
     else:
       completed = line
     return completed, possibilities
   
-  def _commonPrefix(self, str1, str2):
-    for i in range(len(str1)):
-      if not str2.startswith(str1[:i+1]):
-        return str1[:i]
-    return str1
 
   def shell(self, cmd,verbose=0,debug=0,header=''):
+    '''
+    Replacement method to allow shell commands without them blocking.
+    
+    @param cmd: Shell command to execute.
+    @type cmd: string
+    @param verbose: Verbosity
+    @type verbose: integer
+    @param debug: Debug level
+    @type debug: integer
+    @param header: Header to be printed before output
+    @type header: string
+    '''
     stat = 0
     if verbose or debug: print header+cmd
     # flush stdout so we don't mangle python's buffering
@@ -150,7 +228,34 @@ class IterableIPShell:
       input.close()
 
 class ConsoleView(gtk.TextView):
+  '''
+  Specialized text view for console-like workflow.
+
+  @cvar ANSI_COLORS: Mapping of terminal colors to X11 names.
+  @type ANSI_COLORS: dictionary
+
+  @ivar text_buffer: Widget's text buffer.
+  @type text_buffer: gtk.TextBuffer
+  @ivar color_pat: Regex of terminal color pattern
+  @type color_pat: _sre.SRE_Pattern
+  @ivar mark: Scroll mark for automatic scrolling on input.
+  @type mark: gtk.TextMark
+  @ivar line_start: Start of command line mark.
+  @type line_start: gtk.TextMark
+  '''
+  ANSI_COLORS =  {'0;30': 'Black',     '0;31': 'Red',
+                  '0;32': 'Green',     '0;33': 'Brown',
+                  '0;34': 'Blue',      '0;35': 'Purple',
+                  '0;36': 'Cyan',      '0;37': 'LightGray',
+                  '1;30': 'DarkGray',  '1;31': 'DarkRed',
+                  '1;32': 'SeaGreen',  '1;33': 'Yellow',
+                  '1;34': 'LightBlue', '1;35': 'MediumPurple',
+                  '1;36': 'LightCyan', '1;37': 'White'}
+
   def __init__(self):
+    '''
+    Initialize console view.
+    '''
     gtk.TextView.__init__(self)
     self.modify_font(pango.FontDescription('Mono'))
     self.set_cursor_visible(True)
@@ -158,9 +263,9 @@ class ConsoleView(gtk.TextView):
     self.mark = self.text_buffer.create_mark('scroll_mark', 
                                              self.text_buffer.get_end_iter(),
                                              False)
-    for code in ansi_colors:
+    for code in self.ANSI_COLORS:
       self.text_buffer.create_tag(code, 
-                                  foreground=ansi_colors[code], 
+                                  foreground=self.ANSI_COLORS[code], 
                                   weight=700)
     self.text_buffer.create_tag('0')
     self.text_buffer.create_tag('notouch', editable=False)
@@ -169,9 +274,16 @@ class ConsoleView(gtk.TextView):
         self.text_buffer.create_mark('line_start', 
                                      self.text_buffer.get_end_iter(), True)
     self.connect('key-press-event', self._onKeypress)
-    self.last_cursor_pos = 0
     
   def write(self, text, editable=False):
+    '''
+    Write given text to buffer.
+    
+    @param text: Text to append.
+    @type text: string
+    @param editable: If true, added text is editable.
+    @type editable: boolean
+    '''
     segments = self.color_pat.split(text)
     segment = segments.pop(0)
     start_mark = self.text_buffer.create_mark(None, 
@@ -194,21 +306,46 @@ class ConsoleView(gtk.TextView):
     self.scroll_mark_onscreen(self.mark)
 
   def showPrompt(self, prompt):
+    '''
+    Prints prompt at start of line.
+    
+    @param prompt: Prompt to print.
+    @type prompt: string
+    '''
     self.write(prompt)
     self.text_buffer.move_mark(self.line_start,self.text_buffer.get_end_iter())
 
   def changeLine(self, text):
+    '''
+    Replace currently entered command line with given text.
+    
+    @param text: Text to use as replacement.
+    @type text: string
+    '''
     iter = self.text_buffer.get_iter_at_mark(self.line_start)
     iter.forward_to_line_end()
     self.text_buffer.delete(self.text_buffer.get_iter_at_mark(self.line_start), iter)
     self.write(text, True)
 
   def getCurrentLine(self):
-    rv = self.text_buffer.get_slice(self.text_buffer.get_iter_at_mark(self.line_start),
-                                    self.text_buffer.get_end_iter(), False)
+    '''
+    Get text in current command line.
+    
+    @return: Text of current command line.
+    @rtype: string
+    '''
+    rv = self.text_buffer.get_slice(
+      self.text_buffer.get_iter_at_mark(self.line_start),
+      self.text_buffer.get_end_iter(), False)
     return rv
 
   def showReturned(self, text):
+    '''
+    Show returned text from last command and print new prompt.
+    
+    @param text: Text to show.
+    @type text: string
+    '''
     iter = self.text_buffer.get_iter_at_mark(self.line_start)
     iter.forward_to_line_end()
     self.text_buffer.apply_tag_by_name(
@@ -222,7 +359,20 @@ class ConsoleView(gtk.TextView):
     self.text_buffer.move_mark(self.line_start,self.text_buffer.get_end_iter())
     self.text_buffer.place_cursor(self.text_buffer.get_end_iter())
 
-  def _onKeypress(self, obj, event):
+  def _onKeypress(self, widget, event):
+    '''
+    Key press callback used for correcting behavior for console-like 
+    interfaces. For example 'home' should go to prompt, not to begining of
+    line.
+    
+    @param widget: Widget that key press accored in.
+    @type widget: gtk.Widget
+    @param event: Event object
+    @type event: gtk.gdk.Event
+    
+    @return: Return True if event should not trickle.
+    @rtype: boolean
+    '''
     insert_mark = self.text_buffer.get_insert()
     insert_iter = self.text_buffer.get_iter_at_mark(insert_mark)
     selection_mark = self.text_buffer.get_selection_bound()
@@ -255,7 +405,14 @@ class ConsoleView(gtk.TextView):
       self.text_buffer.move_mark(selection_mark, start_iter)             
 
 class IPythonView(ConsoleView, IterableIPShell):
+  '''
+  Sub-class of both modified IPython shell and L{ConsoleView} this makes
+  a GTK+ IPython console.
+  '''
   def __init__(self):
+    '''
+    Initialize. Redirect I/O to console.
+    '''
     ConsoleView.__init__(self)
     self.cout = StringIO()
     IterableIPShell.__init__(self, cout=self.cout,cerr=self.cout, 
@@ -267,12 +424,33 @@ class IPythonView(ConsoleView, IterableIPShell):
     self.interrupt = False
 
   def raw_input(self, prompt=''):
+    '''
+    Custom raw_input() replacement. Get's current line from console buffer.
+    
+    @param prompt: Prompt to print. Here for compatability as replacement.
+    @type prompt: string
+    
+    @return: The current command line text.
+    @rtype: string
+    '''
     if self.interrupt:
       self.interrupt = False
       raise KeyboardInterrupt
     return self.getCurrentLine()
 
   def keyPress(self, widget, event):
+    '''
+    Key press callback with plenty of shell goodness, like history, 
+    autocompletions, etc.
+    
+    @param widget: Widget that key press occured in.
+    @type widget: gtk.Widget
+    @param event: Event object.
+    @type event: gtk.gdk.Event
+    
+    @return: True if event should not trickle.
+    @rtype: boolean
+    '''
     if event.state & gtk.gdk.CONTROL_MASK and event.keyval == 99:
       self.interrupt = True
       self._processLine()
@@ -300,6 +478,9 @@ class IPythonView(ConsoleView, IterableIPShell):
       return True
 
   def _processLine(self):
+    '''
+    Process current command line.
+    '''
     self.history_pos = 0
     self.execute()
     rv = self.cout.getvalue()
